package com.muyu.common.core.utils.file;

import com.muyu.common.core.utils.StringUtils;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.ArrayUtils;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import static com.muyu.common.core.constant.SplitConstants.ADDITION_STR;
import static com.muyu.common.core.constant.SplitConstants.EMPTY_STR;

/**
 * 文件处理工具类
 *
 * @author muyu
 */
@Log4j2
public class FileUtils {
    /**
     * 字符常量：斜杠 {@code '/'}
     */
    public static final char SLASH = '/';

    /**
     * 字符常量：反斜杠 {@code '\\'}
     */
    public static final char BACKSLASH = '\\';

    /**
     * 文件名称正则
     */
    public final static String FILENAME_PATTERN = "[a-zA-Z0-9_\\-\\|\\.\\u4e00-\\u9fa5]+";

    /**
     * 上级目录
     */
    private final static String LAST_PATH = "..";

    /**
     * 请求对象
     */
    private final static String AGENT_KEY = "USER-AGENT";
    /**
     * 微软IE
     */
    private final static String MSIE = "MSIE";

    /**
     * 火狐浏览器
     */
    private final static String FIREFOX = "Firefox";

    /**
     * 谷歌浏览器
     */
    private final static String CHROME = "Chrome";

    /**
     * 文件模板描述
     */
    private final static String CONTENT_DISPOSITION_VALUE_TEMPLATE = "attachment; filename={};filename*=utf-8''{}";
    /**
     * 描述
     */
    private final static String CONTENT_DISPOSITION = "Content-disposition";

    /**
     * 编码
     */
    private final static String CONTENT_ENCODING = "Content-encoding";

    /**
     * 百分号编码
     */
    private final static String PERCENT_SIGN_ENCODE = "\\+";

    /**
     * 百分号解码
     */
    private final static String PERCENT_SIGN_DECODE = "%20";

    /**
     * 输出指定文件的byte数组
     *
     * @param filePath 文件路径
     * @param os       输出流
     */
    public static void writeBytes (String filePath, OutputStream os) throws IOException {
        FileInputStream fis = null;
        try {
            File file = new File(filePath);
            if (!file.exists()) {
                throw new FileNotFoundException(filePath);
            }
            fis = new FileInputStream(file);
            byte[] b = new byte[1024];
            int length;
            while ((length = fis.read(b)) > 0) {
                os.write(b, 0, length);
            }
        } catch (IOException e) {
            throw e;
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e1) {
                    log.error(e1);
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                    log.error(e1);
                }
            }
        }
    }

    /**
     * 删除文件
     *
     * @param filePath 文件
     */
    public static boolean deleteFile (String filePath) {
        boolean flag = false;
        File file = new File(filePath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            flag = file.delete();
        }
        return flag;
    }

    /**
     * 文件名称验证
     *
     * @param filename 文件名称
     *
     * @return true 正常 false 非法
     */
    public static boolean isValidFilename (String filename) {
        return filename.matches(FILENAME_PATTERN);
    }

    /**
     * 检查文件是否可下载
     *
     * @param resource 需要下载的文件
     *
     * @return true 正常 false 非法
     */
    public static boolean checkAllowDownload (String resource) {
        // 禁止目录上跳级别
        if (StringUtils.contains(resource, LAST_PATH)) {
            return false;
        }
        // 判断是否在允许下载的文件规则内
        return ArrayUtils.contains(MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION, FileTypeUtils.getFileType(resource));
    }

    /**
     * 下载文件名重新编码
     *
     * @param request  请求对象
     * @param fileName 文件名
     *
     * @return 编码后的文件名
     */
    public static String setFileDownloadHeader (HttpServletRequest request, String fileName) {
        final String agent = request.getHeader(AGENT_KEY);
        String filename = fileName;
        if (agent.contains(MSIE)) {
            // IE浏览器
            filename = URLEncoder.encode(filename, StandardCharsets.UTF_8);
            filename = filename.replace(ADDITION_STR, EMPTY_STR);
        } else if (agent.contains(FIREFOX)) {
            // 火狐浏览器
            filename = new String(fileName.getBytes(), StandardCharsets.ISO_8859_1);
        } else if (agent.contains(CHROME)) {
            // google浏览器
            filename = URLEncoder.encode(filename, StandardCharsets.UTF_8);
        } else {
            // 其它浏览器
            filename = URLEncoder.encode(filename, StandardCharsets.UTF_8);
        }
        return filename;
    }

    /**
     * 返回文件名
     *
     * @param filePath 文件
     *
     * @return 文件名
     */
    public static String getName (String filePath) {
        if (null == filePath) {
            return null;
        }
        int len = filePath.length();
        if (0 == len) {
            return filePath;
        }
        if (isFileSeparator(filePath.charAt(len - 1))) {
            // 以分隔符结尾的去掉结尾分隔符
            len--;
        }

        int begin = 0;
        char c;
        for (int i = len - 1 ; i > -1 ; i--) {
            c = filePath.charAt(i);
            if (isFileSeparator(c)) {
                // 查找最后一个路径分隔符（/或者\）
                begin = i + 1;
                break;
            }
        }

        return filePath.substring(begin, len);
    }

    /**
     * 是否为Windows或者Linux（Unix）文件分隔符<br>
     * Windows平台下分隔符为\，Linux（Unix）为/
     *
     * @param c 字符
     *
     * @return 是否为Windows或者Linux（Unix）文件分隔符
     */
    public static boolean isFileSeparator (char c) {
        return SLASH == c || BACKSLASH == c;
    }

    /**
     * 下载文件名重新编码
     *
     * @param response     响应对象
     * @param realFileName 真实文件名
     */
    public static void setAttachmentResponseHeader (HttpServletResponse response, String realFileName) throws UnsupportedEncodingException {
        String percentEncodedFileName = percentEncode(realFileName);
        String contentDispositionValue = StringUtils.format(CONTENT_DISPOSITION_VALUE_TEMPLATE, percentEncodedFileName, percentEncodedFileName);
        response.setHeader(CONTENT_DISPOSITION, contentDispositionValue);
        response.setHeader(CONTENT_ENCODING, percentEncodedFileName);
    }

    /**
     * 百分号编码工具方法
     *
     * @param s 需要百分号编码的字符串
     *
     * @return 百分号编码后的字符串
     */
    public static String percentEncode (String s) {
        String encode = URLEncoder.encode(s, StandardCharsets.UTF_8);
        return encode.replaceAll(PERCENT_SIGN_ENCODE, PERCENT_SIGN_DECODE);
    }
}
