package com.muyu.system.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.DataPageResp;
import com.muyu.common.core.web.page.PageQueryModel;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.system.domain.SysPost;
import com.muyu.system.domain.model.SysPostEditModel;
import com.muyu.system.domain.model.SysPostListModel;
import com.muyu.system.domain.model.SysPostPageQueryModel;
import com.muyu.system.domain.model.SysPostSaveModel;
import com.muyu.system.domain.req.SysPostEditReq;
import com.muyu.system.domain.req.SysPostListReq;
import com.muyu.system.domain.req.SysPostSaveReq;
import com.muyu.system.domain.resp.SysPostListResp;
import com.muyu.system.domain.resp.SysPostMyInfoResp;
import com.muyu.system.service.SysPostService;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 岗位信息操作处理
 *
 * @author muyu
 */
@RestController
@RequestMapping("/post")
public class SysPostController extends BaseController {
    @Autowired
    private SysPostService postService;

    /**
     * 获取岗位列表
     */
    @RequiresPermissions("system:post:list")
    @PostMapping("/list")
    public Result<DataPageResp<SysPostListResp>> list (@RequestBody SysPostListReq postListReq) {
        PageQueryModel<SysPostListModel> sysPostPageQueryModel = postService.getList(SysPostPageQueryModel.reqBuild(postListReq));
        return Result.success(DataPageResp.of(sysPostPageQueryModel.getTotal(),sysPostPageQueryModel.getDataList().stream().map(SysPostListResp::listBuild).toList()));
    }

    @Log(title = "岗位管理", businessType = BusinessType.EXPORT)
    @RequiresPermissions("system:post:export")
    @PostMapping("/export")
    public void export (HttpServletResponse response, SysPostListReq postListReq) {
        // TODO 导出重写
        PageQueryModel<SysPostListModel> sysPostPageQueryModel = postService.getList(SysPostPageQueryModel.reqBuild(postListReq));
        ExcelUtil<SysPostListResp> util = new ExcelUtil<>(SysPostListResp.class);
        util.exportExcel(response, sysPostPageQueryModel.getDataList().stream().map(SysPostListResp::listBuild).toList(), "岗位数据");
    }

    /**
     * 根据岗位编号获取详细信息
     */
    @RequiresPermissions("system:post:query")
    @GetMapping(value = "/{postId}")
    public Result<SysPostMyInfoResp> getInfo (@PathVariable("postId") Long postId) {
        return success(SysPostMyInfoResp.messageBuild(postService.selectPostById(postId)));
    }

    /**
     * 新增岗位
     */
    @RequiresPermissions("system:post:add")
    @Log(title = "岗位管理", businessType = BusinessType.INSERT)
    @PostMapping
    public Result<String> add (@Validated @RequestBody SysPostSaveReq sysPostSaveReq) {
        if (!postService.checkPostNameUnique(SysPostSaveModel.saveBuild(sysPostSaveReq))) {
            return error("新增岗位'" + sysPostSaveReq.getPostName() + "'失败，岗位名称已存在");
        } else if (!postService.checkPostCodeUnique(SysPostSaveModel.saveBuild(sysPostSaveReq))) {
            return error("新增岗位'" + sysPostSaveReq.getPostName() + "'失败，岗位编码已存在");
        }
        postService.insertPost(SysPost.saveBuild(SysPostSaveModel.saveBuild(sysPostSaveReq)));
        return Result.success();
    }

    /**
     * 修改岗位
     */
    @RequiresPermissions("system:post:edit")
    @Log(title = "岗位管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result<String> edit (@Validated @RequestBody SysPostEditReq sysPostEditReq) {
        if (!postService.checkEditPostNameUnique(SysPostEditModel.editBuild(sysPostEditReq))) {
            return error("修改岗位'" + sysPostEditReq.getPostName() + "'失败，岗位名称已存在");
        } else if (!postService.checkEditPostCodeUnique(SysPostEditModel.editBuild(sysPostEditReq))) {
            return error("修改岗位'" + sysPostEditReq.getPostName() + "'失败，岗位编码已存在");
        }
        postService.updatePost(SysPost.editBuild(SysPostEditModel.editBuild(sysPostEditReq)));
        return Result.success();
    }

    /**
     * 删除岗位
     */
    @RequiresPermissions("system:post:remove")
    @Log(title = "岗位管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{postIds}")
    public Result<String> remove (@PathVariable("postIds")List<Long> postIds) {
        postService.deletePostByIds(postIds);
        return Result.success();
    }

    /**
     * 获取岗位选择框列表
     */
    @GetMapping("/optionselect")
    public Result<List<SysPost>> optionselect () {
        List<SysPost> posts = postService.selectPostAll();
        return success(posts);
    }
}
